#include "mex.h"
#include <math.h>

/*
 * pairtest3.c - example of C-MEX file
 *
 */

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  double *x, *l, *k, *kcpy, *lcpy;
  double d;
  int    p[1] = {0};
  int    mrows,ncols, i, j;
  
  /* Check for proper number of arguments. */
  if(nrhs!=2) {
    mexErrMsgTxt("Two inputs required.");
  } else if(nlhs>3) {
    mexErrMsgTxt("Too many output arguments");
  }
  
  /* The second input must be a noncomplex scalar double.*/
  mrows = mxGetM(prhs[1]);
  ncols = mxGetN(prhs[1]);
  if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||
      !(mrows==1 && ncols==1) ) {
    mexErrMsgTxt("Second input must be a noncomplex scalar double.");
  }
  /* The first input must be a scalar double array.*/
  mrows = mxGetM(prhs[0]);
  ncols = mxGetN(prhs[0]);
  if( !mxIsDouble(prhs[0]) || !((mrows>=1 && ncols==1) ||
      (mrows==1 && ncols>=1)) ) {
    mexErrMsgTxt("First input must be a double array.");
  }
  
  /* Convert input array to column array */
  if( mrows < ncols )
    mrows = ncols;

  /* Allocate memory for maximum number of pairings */
  k = (double*)malloc(sizeof(double)*mrows*(mrows-1)/2);
  l = (double*)malloc(sizeof(double)*mrows*(mrows-1)/2);
  
  /* Assign pointers to input. */
  x = mxGetPr(prhs[0]);
  d = mxGetScalar(prhs[1]);

  /* The actual routine */
  for (i=0; i<mrows-1; i++)
    for (j=i+1; j<mrows; j++)
      if (fabs(x[i] - x[j]) < d) {
        k[p[0]] = i;
        l[p[0]] = j;
        p[0] = p[0] + 1;
      }

  /* Create two matrices for the return argument. */
  plhs[0] = mxCreateDoubleMatrix(p[0],1, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(p[0],1, mxREAL);

  /* Copy result to output matrices */
  kcpy = mxGetPr(plhs[0]);
  lcpy = mxGetPr(plhs[1]);
  for(i = 0; i < p[0]; i++) {
    kcpy[i] = k[i];
    lcpy[i] = l[i];
  }

  /* Free locally allocated memory */
  free(k);  free(l);
}
